public class HashTab implements Assoc {
    private String[] ks = new String[65];
    private String[] vs = new String[65];
    private int count = 0;

    private int find(String k) {
        if (k == null) {
            return ks.length - 1;
        }
        int i = k.hashCode() & (ks.length - 2);
        while (ks[i] != null && !ks[i].equals(k)) {
            i = (i + 1) & (ks.length - 2);
        }
        return i;
    }

    public String put(String k, String v) {
        if (k == null) {
            return null;
        }
        int i = find(k);
        String old = vs[i];
        vs[i] = v;
        if (ks[i] == null) {
            ks[i] = k;
            if (++count >= ks.length * 3 / 4) {
                String[] oks = ks, ovs = vs;
                ks = new String[(oks.length << 1) - 1];
                vs = new String[(oks.length << 1) - 1];
                for (int j = 0; j < oks.length; j++) {
                    if (oks[j] != null) {
                        ks[i = find(oks[j])] = oks[j];
                        vs[i] = ovs[j];
                    }
                }
            }
        }
        return old;
    }

    public String get(String k) {
        return vs[find(k)];
    }

    public boolean containsKey(String k) {
        return ks[find(k)] != null;
    }

    public boolean containsValue(String v) {
        for (int i = 0; i < vs.length - 1; i++) {
            if (v == null ? ks[i] != v && vs[i] == v : v.equals(vs[i])) {
                return true;
            }
        }
        return false;
    }

    public String toString() {
        String s = "";
        for (int i = 0; i < ks.length - 1; i++) {
            if (ks[i] != null) {
                s += (s.isEmpty() ? "(" : ", (") + ks[i] + ", " + vs[i] + ")";
            }
        }
        return "{" + s + "}";
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || o.getClass() != HashTab.class) {
            return false;
        }
        HashTab t = (HashTab) o;
        if (count != t.count) {
            return false;
        }
        for (int i = 0; i < ks.length - 1; i++) {
            if (ks[i] != null &&
                    (vs[i] == null ? !t.containsKey(ks[i]) || t.get(ks[i]) != null : !vs[i].equals(t.get(ks[i])))) {
                return false;
            }
        }
        return true;
    }

    public int hashCode() {
        int h = count;
        for (int i = 0; i < ks.length - 1; i++) {
            if (ks[i] != null) {
                h += ks[i].hashCode();
                if (vs[i] != null) {
                    h += vs[i].hashCode();
                }
            }
        }
        return h;
    }
}
