public class TreeAssocData implements AssocData {
    private TADNode root;

    public String put(Data k, String v) {
        if (k != null) {
            if (root != null) {
                return root.put(k, v);
            }
            root = new TADNode(k, v);
        }
        return null;
    }

    public String get(Data k) {
        if (k == null || root == null) {
            return null;
        }
        TADNode node = root.find(k);
        return node == null ? null : node.value();
    }

    public boolean containsKey(Data k) {
        return k != null && root != null && root.find(k) != null;
    }

    @Override
    public String toString() {
        return root == null ? "{}" : "{" + root + "}";
    }

}

class TADNode {
    private Data key;
    private String value;
    private TADNode left, right;

    public TADNode(Data k, String v) {
        key = k;
        value = v;
    }

    public String put(Data k, String v) {
        int cmp = key.compareTo(k);
        if (cmp < 0) {
            if (left != null) {
                return left.put(k, v);
            }
            left = new TADNode(k, v);
        } else if (cmp > 0) {
            if (right != null) {
                return right.put(k, v);
            }
            right = new TADNode(k, v);
        } else {
            String result = value;
            value = v;
            return result;
        }
        return null;
    }

    public TADNode find(Data k) {
        int cmp = key.compareTo(k);
        if (cmp == 0) {
            return this;
        }
        TADNode node = cmp < 0 ? left : right;
        if (node == null) {
            return null;
        }
        return node.find(k);
    }

    public String value() {
        return value;
    }

    @Override
    public String toString() {
        String s = "(" + key + ", " + value + ")";
        if (left != null) {
            s = left + ", " + s;
        }
        if (right != null) {
            s += ", " + right;
        }
        return s;
    }

}
