// An associative data structure holding entries each consisting of a key and a value;
// there is no guarantee that null can be used as key or value, although some implementations may allow us to do so
public interface Assoc {

    // inserts an entry with key k and value v if there was no entry with key k before;
    // substitutes v for the value of an entry with key k if there already was such an entry;
    // returns the old value of this entry or null (if there was no such entry before or the old value was null)
    String put(String k, String v);

    // returns the value of an entry where the key equals k;
    // returns null if there is no such entry (or if the value of the entry is null)
    String get(String k);

    // returns true if there is an entry with a key equal to k
    boolean containsKey(String k);

    // returns true if there is an entry with a value equal to v
    boolean containsValue(String v);

    // inserts an entry with key k and value v if there was no entry with key k before;
    // substitutes v for the value of an entry with key k and value null if there already was such an entry;
    // returns true if a new entry was inserted or v replaced the old value null;
    // has no effect and returns false if there already is an entry with key k and a value different from null
    default boolean putConditional(String k, String v) {
        if (get(k) == null) {
            put(k, v);
            return true;
        }
        return false;
    }

}
