import java.util.Arrays;

public class OrderedTree implements Ordered {
    private OTNode root;
    private int[] cnt = new int[1];

    public OrderedTree() {
    }

    public OrderedTree(String[] elems) {
        for (int i = 1 << 30, mask = (1 << 31) - 1; i > 0; i >>= 1, mask >>= 1) {
            for (int j = i; j <= elems.length; j += i) {
                if ((j & mask) != 0) {
                    add(elems[j - 1]);
                }
            }
        }
    }

    public OrderedTree(Iterable elems) {
        for (String e : elems) {
            add(e);
        }
    }

    public void add(String e) {
        cnt[0]++;
        if (root != null) {
            OTNode n = root.add(e);
            if (n != null) {
                root = n;
            }
        } else {
            root = new OTNode(e);
        }
    }

    public int contains(String e) {
        if (root == null) {
            return 0;
        }
        return root.contains(e);
    }

    public int removeAll(String e) {
        int c = contains(e);
        if (c > 0) {
            Iterator iter = iterator();
            clear();
            while (iter.hasNext()) {
                String s = iter.next();
                if (e == null ? s != null : !e.equals(s)) {
                    add(s);
                }
            }
        }
        return c;
    }

    public void clear() {
        root = null;
        cnt[0] = 0;
    }

    public int size() {
        return cnt[0];
    }

    public String[] toArray() {
        String[] array = new String[size()];
        toArray(array);
        return array;
    }

    public int toArray(String[] array) {
        return root == null ? 0 : root.fill(array, 0);
    }

    public Iterator iterator() {
        OTIter iter = new OTIter();
        if (root != null) {
            root.iter(iter, false);
        }
        return iter;
    }

    public String toString() {
        if (root == null) {
            return "[]";
        } else {
            return root.toString();
        }
    }

    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if (o == null || o.getClass() != getClass()) {
            return false;
        }
        OrderedTree t = (OrderedTree) o;
        return Arrays.equals(t.toArray(), toArray());
    }

    public int hashCode() {
        return 57 - Arrays.hashCode(toArray());
    }


    public int depth() {
        if (root == null) {
            return 0;
        }
        return root.depth();
    }
}
